@@ -103,11 +103,14 @@ final readonly class PhpWarningTriggered implements Event
         return $this->ignoredByBaseline;
     }
 
+    /**
+     * @return non-empty-string
+     */
     public function asString(): string
     {
         $message = $this->message;
 
-        if (!empty($message)) {
+        if ($message !== '') {
             $message = PHP_EOL . $message;
         }
 
