@@ -12,12 +12,14 @@ namespace PHPUnit\Metadata\Parser;
 use const JSON_THROW_ON_ERROR;
 use function assert;
 use function class_exists;
+use function is_numeric;
 use function json_decode;
 use function method_exists;
 use function sprintf;
 use function str_starts_with;
 use function strtolower;
 use function trim;
+use Error;
 use PHPUnit\Event\Facade as EventFacade;
 use PHPUnit\Framework\Attributes\After;
 use PHPUnit\Framework\Attributes\AfterClass;
@@ -26,8 +28,11 @@ use PHPUnit\Framework\Attributes\BackupStaticProperties;
 use PHPUnit\Framework\Attributes\Before;
 use PHPUnit\Framework\Attributes\BeforeClass;
 use PHPUnit\Framework\Attributes\CoversClass;
+use PHPUnit\Framework\Attributes\CoversClassesThatExtendClass;
+use PHPUnit\Framework\Attributes\CoversClassesThatImplementInterface;
 use PHPUnit\Framework\Attributes\CoversFunction;
 use PHPUnit\Framework\Attributes\CoversMethod;
+use PHPUnit\Framework\Attributes\CoversNamespace;
 use PHPUnit\Framework\Attributes\CoversNothing;
 use PHPUnit\Framework\Attributes\CoversTrait;
 use PHPUnit\Framework\Attributes\DataProvider;
@@ -48,11 +53,13 @@ use PHPUnit\Framework\Attributes\ExcludeStaticPropertyFromBackup;
 use PHPUnit\Framework\Attributes\Group;
 use PHPUnit\Framework\Attributes\IgnoreDeprecations;
 use PHPUnit\Framework\Attributes\IgnorePhpunitDeprecations;
+use PHPUnit\Framework\Attributes\IgnorePhpunitWarnings;
 use PHPUnit\Framework\Attributes\Large;
 use PHPUnit\Framework\Attributes\Medium;
 use PHPUnit\Framework\Attributes\PostCondition;
 use PHPUnit\Framework\Attributes\PreCondition;
 use PHPUnit\Framework\Attributes\PreserveGlobalState;
+use PHPUnit\Framework\Attributes\RequiresEnvironmentVariable;
 use PHPUnit\Framework\Attributes\RequiresFunction;
 use PHPUnit\Framework\Attributes\RequiresMethod;
 use PHPUnit\Framework\Attributes\RequiresOperatingSystem;
@@ -68,17 +75,24 @@ use PHPUnit\Framework\Attributes\RunTestsInSeparateProcesses;
 use PHPUnit\Framework\Attributes\Small;
 use PHPUnit\Framework\Attributes\Test;
 use PHPUnit\Framework\Attributes\TestDox;
+use PHPUnit\Framework\Attributes\TestDoxFormatter;
+use PHPUnit\Framework\Attributes\TestDoxFormatterExternal;
 use PHPUnit\Framework\Attributes\TestWith;
 use PHPUnit\Framework\Attributes\TestWithJson;
 use PHPUnit\Framework\Attributes\Ticket;
 use PHPUnit\Framework\Attributes\UsesClass;
+use PHPUnit\Framework\Attributes\UsesClassesThatExtendClass;
+use PHPUnit\Framework\Attributes\UsesClassesThatImplementInterface;
 use PHPUnit\Framework\Attributes\UsesFunction;
 use PHPUnit\Framework\Attributes\UsesMethod;
+use PHPUnit\Framework\Attributes\UsesNamespace;
 use PHPUnit\Framework\Attributes\UsesTrait;
+use PHPUnit\Framework\Attributes\WithEnvironmentVariable;
 use PHPUnit\Framework\Attributes\WithoutErrorHandler;
+use PHPUnit\Metadata\InvalidAttributeException;
 use PHPUnit\Metadata\Metadata;
 use PHPUnit\Metadata\MetadataCollection;
-use PHPUnit\Metadata\Version\ConstraintRequirement;
+use PHPUnit\Metadata\Version\Requirement;
 use ReflectionClass;
 use ReflectionMethod;
 
@@ -96,9 +110,14 @@ final readonly class AttributeParser implements Parser
     {
         assert(class_exists($className));
 
-        $result = [];
+        $reflector = new ReflectionClass($className);
+        $result    = [];
 
-        foreach ((new ReflectionClass($className))->getAttributes() as $attribute) {
+        $small  = false;
+        $medium = false;
+        $large  = false;
+
+        foreach ($reflector->getAttributes() as $attribute) {
             if (!str_starts_with($attribute->getName(), 'PHPUnit\\Framework\\Attributes\\')) {
                 continue;
             }
@@ -107,7 +126,17 @@ final readonly class AttributeParser implements Parser
                 continue;
             }
 
-            $attributeInstance = $attribute->newInstance();
+            try {
+                $attributeInstance = $attribute->newInstance();
+            } catch (Error $e) {
+                throw new InvalidAttributeException(
+                    $attribute->getName(),
+                    'class ' . $className,
+                    $reflector->getFileName(),
+                    $reflector->getStartLine(),
+                    $e->getMessage(),
+                );
+            }
 
             switch ($attribute->getName()) {
                 case BackupGlobals::class:
@@ -124,6 +153,13 @@ final readonly class AttributeParser implements Parser
 
                     break;
 
+                case CoversNamespace::class:
+                    assert($attributeInstance instanceof CoversNamespace);
+
+                    $result[] = Metadata::coversNamespace($attributeInstance->namespace());
+
+                    break;
+
                 case CoversClass::class:
                     assert($attributeInstance instanceof CoversClass);
 
@@ -131,6 +167,20 @@ final readonly class AttributeParser implements Parser
 
                     break;
 
+                case CoversClassesThatExtendClass::class:
+                    assert($attributeInstance instanceof CoversClassesThatExtendClass);
+
+                    $result[] = Metadata::coversClassesThatExtendClass($attributeInstance->className());
+
+                    break;
+
+                case CoversClassesThatImplementInterface::class:
+                    assert($attributeInstance instanceof CoversClassesThatImplementInterface);
+
+                    $result[] = Metadata::coversClassesThatImplementInterface($attributeInstance->interfaceName());
+
+                    break;
+
                 case CoversTrait::class:
                     assert($attributeInstance instanceof CoversTrait);
 
@@ -196,20 +246,58 @@ final readonly class AttributeParser implements Parser
 
                     break;
 
-                case Large::class:
-                    $result[] = Metadata::groupOnClass('large');
+                case Small::class:
+                    if (!$medium && !$large) {
+                        $result[] = Metadata::groupOnClass('small');
+
+                        $small = true;
+                    } else {
+                        EventFacade::emitter()->testRunnerTriggeredPhpunitWarning(
+                            sprintf(
+                                '#[Small] cannot be combined with #[Medium] or #[Large] for %s',
+                                $this->testAsString($className),
+                            ),
+                        );
+                    }
 
                     break;
 
                 case Medium::class:
-                    $result[] = Metadata::groupOnClass('medium');
+                    if (!$small && !$large) {
+                        $result[] = Metadata::groupOnClass('medium');
+
+                        $medium = true;
+                    } else {
+                        EventFacade::emitter()->testRunnerTriggeredPhpunitWarning(
+                            sprintf(
+                                '#[Medium] cannot be combined with #[Small] or #[Large] for %s',
+                                $this->testAsString($className),
+                            ),
+                        );
+                    }
+
+                    break;
+
+                case Large::class:
+                    if (!$small && !$medium) {
+                        $result[] = Metadata::groupOnClass('large');
+
+                        $large = true;
+                    } else {
+                        EventFacade::emitter()->testRunnerTriggeredPhpunitWarning(
+                            sprintf(
+                                '#[Large] cannot be combined with #[Small] or #[Medium] for %s',
+                                $this->testAsString($className),
+                            ),
+                        );
+                    }
 
                     break;
 
                 case IgnoreDeprecations::class:
                     assert($attributeInstance instanceof IgnoreDeprecations);
 
-                    $result[] = Metadata::ignoreDeprecationsOnClass();
+                    $result[] = Metadata::ignoreDeprecationsOnClass($attributeInstance->messagePattern());
 
                     break;
 
@@ -262,8 +350,9 @@ final readonly class AttributeParser implements Parser
                     assert($attributeInstance instanceof RequiresPhp);
 
                     $result[] = Metadata::requiresPhpOnClass(
-                        ConstraintRequirement::from(
+                        $this->requirement(
                             $attributeInstance->versionRequirement(),
+                            $className,
                         ),
                     );
 
@@ -276,7 +365,10 @@ final readonly class AttributeParser implements Parser
                     $versionRequirement = $attributeInstance->versionRequirement();
 
                     if ($versionRequirement !== null) {
-                        $versionConstraint = ConstraintRequirement::from($versionRequirement);
+                        $versionConstraint = $this->requirement(
+                            $versionRequirement,
+                            $className,
+                        );
                     }
 
                     $result[] = Metadata::requiresPhpExtensionOnClass(
@@ -290,8 +382,9 @@ final readonly class AttributeParser implements Parser
                     assert($attributeInstance instanceof RequiresPhpunit);
 
                     $result[] = Metadata::requiresPhpunitOnClass(
-                        ConstraintRequirement::from(
+                        $this->requirement(
                             $attributeInstance->versionRequirement(),
+                            $className,
                         ),
                     );
 
@@ -306,6 +399,26 @@ final readonly class AttributeParser implements Parser
 
                     break;
 
+                case RequiresEnvironmentVariable::class:
+                    assert($attributeInstance instanceof RequiresEnvironmentVariable);
+
+                    $result[] = Metadata::requiresEnvironmentVariableOnClass(
+                        $attributeInstance->environmentVariableName(),
+                        $attributeInstance->value(),
+                    );
+
+                    break;
+
+                case WithEnvironmentVariable::class:
+                    assert($attributeInstance instanceof WithEnvironmentVariable);
+
+                    $result[] = Metadata::withEnvironmentVariableOnClass(
+                        $attributeInstance->environmentVariableName(),
+                        $attributeInstance->value(),
+                    );
+
+                    break;
+
                 case RequiresSetting::class:
                     assert($attributeInstance instanceof RequiresSetting);
 
@@ -326,11 +439,6 @@ final readonly class AttributeParser implements Parser
 
                     break;
 
-                case Small::class:
-                    $result[] = Metadata::groupOnClass('small');
-
-                    break;
-
                 case TestDox::class:
                     assert($attributeInstance instanceof TestDox);
 
@@ -345,6 +453,13 @@ final readonly class AttributeParser implements Parser
 
                     break;
 
+                case UsesNamespace::class:
+                    assert($attributeInstance instanceof UsesNamespace);
+
+                    $result[] = Metadata::usesNamespace($attributeInstance->namespace());
+
+                    break;
+
                 case UsesClass::class:
                     assert($attributeInstance instanceof UsesClass);
 
@@ -352,6 +467,20 @@ final readonly class AttributeParser implements Parser
 
                     break;
 
+                case UsesClassesThatExtendClass::class:
+                    assert($attributeInstance instanceof UsesClassesThatExtendClass);
+
+                    $result[] = Metadata::usesClassesThatExtendClass($attributeInstance->className());
+
+                    break;
+
+                case UsesClassesThatImplementInterface::class:
+                    assert($attributeInstance instanceof UsesClassesThatImplementInterface);
+
+                    $result[] = Metadata::usesClassesThatImplementInterface($attributeInstance->interfaceName());
+
+                    break;
+
                 case UsesTrait::class:
                     assert($attributeInstance instanceof UsesTrait);
 
@@ -390,9 +519,10 @@ final readonly class AttributeParser implements Parser
         assert(class_exists($className));
         assert(method_exists($className, $methodName));
 
-        $result = [];
+        $reflector = new ReflectionMethod($className, $methodName);
+        $result    = [];
 
-        foreach ((new ReflectionMethod($className, $methodName))->getAttributes() as $attribute) {
+        foreach ($reflector->getAttributes() as $attribute) {
             if (!str_starts_with($attribute->getName(), 'PHPUnit\\Framework\\Attributes\\')) {
                 continue;
             }
@@ -401,7 +531,17 @@ final readonly class AttributeParser implements Parser
                 continue;
             }
 
-            $attributeInstance = $attribute->newInstance();
+            try {
+                $attributeInstance = $attribute->newInstance();
+            } catch (Error $e) {
+                throw new InvalidAttributeException(
+                    $attribute->getName(),
+                    'method ' . $className . '::' . $methodName . '()',
+                    $reflector->getFileName(),
+                    $reflector->getStartLine(),
+                    $e->getMessage(),
+                );
+            }
 
             switch ($attribute->getName()) {
                 case After::class:
@@ -454,14 +594,14 @@ final readonly class AttributeParser implements Parser
                 case DataProvider::class:
                     assert($attributeInstance instanceof DataProvider);
 
-                    $result[] = Metadata::dataProvider($className, $attributeInstance->methodName());
+                    $result[] = Metadata::dataProvider($className, $attributeInstance->methodName(), $attributeInstance->validateArgumentCount());
 
                     break;
 
                 case DataProviderExternal::class:
                     assert($attributeInstance instanceof DataProviderExternal);
 
-                    $result[] = Metadata::dataProvider($attributeInstance->className(), $attributeInstance->methodName());
+                    $result[] = Metadata::dataProvider($attributeInstance->className(), $attributeInstance->methodName(), $attributeInstance->validateArgumentCount());
 
                     break;
 
@@ -564,7 +704,7 @@ final readonly class AttributeParser implements Parser
                 case IgnoreDeprecations::class:
                     assert($attributeInstance instanceof IgnoreDeprecations);
 
-                    $result[] = Metadata::ignoreDeprecationsOnMethod();
+                    $result[] = Metadata::ignoreDeprecationsOnMethod($attributeInstance->messagePattern());
 
                     break;
 
@@ -631,8 +771,10 @@ final readonly class AttributeParser implements Parser
                     assert($attributeInstance instanceof RequiresPhp);
 
                     $result[] = Metadata::requiresPhpOnMethod(
-                        ConstraintRequirement::from(
+                        $this->requirement(
                             $attributeInstance->versionRequirement(),
+                            $className,
+                            $methodName,
                         ),
                     );
 
@@ -645,7 +787,11 @@ final readonly class AttributeParser implements Parser
                     $versionRequirement = $attributeInstance->versionRequirement();
 
                     if ($versionRequirement !== null) {
-                        $versionConstraint = ConstraintRequirement::from($versionRequirement);
+                        $versionConstraint = $this->requirement(
+                            $versionRequirement,
+                            $className,
+                            $methodName,
+                        );
                     }
 
                     $result[] = Metadata::requiresPhpExtensionOnMethod(
@@ -659,8 +805,10 @@ final readonly class AttributeParser implements Parser
                     assert($attributeInstance instanceof RequiresPhpunit);
 
                     $result[] = Metadata::requiresPhpunitOnMethod(
-                        ConstraintRequirement::from(
+                        $this->requirement(
                             $attributeInstance->versionRequirement(),
+                            $className,
+                            $methodName,
                         ),
                     );
 
@@ -675,6 +823,26 @@ final readonly class AttributeParser implements Parser
 
                     break;
 
+                case RequiresEnvironmentVariable::class:
+                    assert($attributeInstance instanceof RequiresEnvironmentVariable);
+
+                    $result[] = Metadata::requiresEnvironmentVariableOnMethod(
+                        $attributeInstance->environmentVariableName(),
+                        $attributeInstance->value(),
+                    );
+
+                    break;
+
+                case WithEnvironmentVariable::class:
+                    assert($attributeInstance instanceof WithEnvironmentVariable);
+
+                    $result[] = Metadata::withEnvironmentVariableOnMethod(
+                        $attributeInstance->environmentVariableName(),
+                        $attributeInstance->value(),
+                    );
+
+                    break;
+
                 case RequiresSetting::class:
                     assert($attributeInstance instanceof RequiresSetting);
 
@@ -702,6 +870,20 @@ final readonly class AttributeParser implements Parser
 
                     break;
 
+                case TestDoxFormatter::class:
+                    assert($attributeInstance instanceof TestDoxFormatter);
+
+                    $result[] = Metadata::testDoxFormatter($className, $attributeInstance->methodName());
+
+                    break;
+
+                case TestDoxFormatterExternal::class:
+                    assert($attributeInstance instanceof TestDoxFormatterExternal);
+
+                    $result[] = Metadata::testDoxFormatter($attributeInstance->className(), $attributeInstance->methodName());
+
+                    break;
+
                 case TestWith::class:
                     assert($attributeInstance instanceof TestWith);
 
@@ -731,6 +913,13 @@ final readonly class AttributeParser implements Parser
 
                     $result[] = Metadata::withoutErrorHandler();
 
+                    break;
+
+                case IgnorePhpunitWarnings::class:
+                    assert($attributeInstance instanceof IgnorePhpunitWarnings);
+
+                    $result[] = Metadata::ignorePhpunitWarnings($attributeInstance->messagePattern());
+
                     break;
             }
         }
@@ -762,17 +951,51 @@ final readonly class AttributeParser implements Parser
             return false;
         }
 
-        EventFacade::emitter()->testRunnerTriggeredWarning(
+        EventFacade::emitter()->testRunnerTriggeredPhpunitWarning(
             sprintf(
-                'Group name "%s" is not allowed for %s %s%s%s',
+                'Group name "%s" is not allowed for %s',
                 $_groupName,
-                $testMethodName !== null ? 'method' : 'class',
-                $testClassName,
-                $testMethodName !== null ? '::' : '',
-                $testMethodName !== null ? $testMethodName : '',
+                $this->testAsString($testClassName, $testMethodName),
             ),
         );
 
         return true;
     }
+
+    /**
+     * @param non-empty-string  $versionRequirement
+     * @param class-string      $testClassName
+     * @param ?non-empty-string $testMethodName
+     */
+    private function requirement(string $versionRequirement, string $testClassName, ?string $testMethodName = null): Requirement
+    {
+        if (is_numeric(trim($versionRequirement))) {
+            EventFacade::emitter()->testRunnerTriggeredPhpunitDeprecation(
+                sprintf(
+                    'Test %s has attribute with version constraint string argument without explicit version comparison operator ("%s")',
+                    $this->testAsString($testClassName, $testMethodName),
+                    $versionRequirement,
+                ),
+            );
+        }
+
+        return Requirement::from($versionRequirement);
+    }
+
+    /**
+     * @param class-string      $testClassName
+     * @param ?non-empty-string $testMethodName
+     *
+     * @return non-empty-string
+     */
+    private function testAsString(string $testClassName, ?string $testMethodName = null): string
+    {
+        return sprintf(
+            '%s %s%s%s',
+            $testMethodName !== null ? 'method' : 'class',
+            $testClassName,
+            $testMethodName !== null ? '::' : '',
+            $testMethodName !== null ? $testMethodName : '',
+        );
+    }
 }
