@@ -41,6 +41,7 @@ use PHPUnit\TestFixture\Metadata\Attribute\IgnoreDeprecationsClassTest;
 use PHPUnit\TestFixture\Metadata\Attribute\IgnoreDeprecationsMethodTest;
 use PHPUnit\TestFixture\Metadata\Attribute\IgnorePhpunitDeprecationsClassTest;
 use PHPUnit\TestFixture\Metadata\Attribute\IgnorePhpunitDeprecationsMethodTest;
+use PHPUnit\TestFixture\Metadata\Attribute\IgnorePhpunitWarningsTest;
 use PHPUnit\TestFixture\Metadata\Attribute\LargeTest;
 use PHPUnit\TestFixture\Metadata\Attribute\MediumTest;
 use PHPUnit\TestFixture\Metadata\Attribute\NonPhpunitAttributeTest;
@@ -834,6 +835,15 @@ abstract class AttributeParserTestCase extends TestCase
         $this->assertTrue($metadata->asArray()[0]->isIgnorePhpunitDeprecations());
     }
 
+    #[TestDox('Parses #[IgnorePhpunitWarnings] attribute on method')]
+    public function test_parses_IgnorePhpunitWarnings_attribute_on_method(): void
+    {
+        $metadata = $this->parser()->forMethod(IgnorePhpunitWarningsTest::class, 'testOne')->isIgnorePhpunitWarnings();
+
+        $this->assertCount(1, $metadata);
+        $this->assertTrue($metadata->asArray()[0]->isIgnorePhpunitWarnings());
+    }
+
     #[TestDox('Parses #[PostCondition] attribute on method')]
     public function test_parses_PostCondition_attribute_on_method(): void
     {
@@ -1088,6 +1098,27 @@ abstract class AttributeParserTestCase extends TestCase
         $this->assertSame('text', $metadata->asArray()[0]->text());
     }
 
+    #[TestDox('Parses #[TestDoxFormatter] attribute on method')]
+    public function test_parses_TestDoxFormatter_attribute_on_method(): void
+    {
+        $metadata = $this->parser()->forMethod(TestDoxTest::class, 'testTwo')->isTestDoxFormatter();
+
+        $this->assertCount(1, $metadata);
+        $this->assertTrue($metadata->asArray()[0]->isTestDoxFormatter());
+        $this->assertSame('methodName', $metadata->asArray()[0]->methodName());
+    }
+
+    #[TestDox('Parses #[TestDoxFormatterExternal] attribute on method')]
+    public function test_parses_TestDoxFormatterExternal_attribute_on_method(): void
+    {
+        $metadata = $this->parser()->forMethod(TestDoxTest::class, 'testThree')->isTestDoxFormatter();
+
+        $this->assertCount(1, $metadata);
+        $this->assertTrue($metadata->asArray()[0]->isTestDoxFormatter());
+        $this->assertSame('ClassName', $metadata->asArray()[0]->className());
+        $this->assertSame('methodName', $metadata->asArray()[0]->methodName());
+    }
+
     #[TestDox('Parses #[TestWith] attribute on method')]
     public function test_parses_TestWith_attribute_on_method(): void
     {
